"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = transform;
exports.transform = transform;
exports.preprendTransformerToOptions = preprendTransformerToOptions;
const stream_1 = require("stream");
const debug_1 = __importDefault(require("../helpers/debug"));
const template_1 = require("../template");
function transform(configuration, filepath) {
    if (!filepath.match(".feature$")) {
        return new stream_1.PassThrough();
    }
    (0, debug_1.default)(`compiling ${filepath}`);
    let buffer = Buffer.alloc(0);
    return new stream_1.Transform({
        transform(chunk, encoding, done) {
            buffer = Buffer.concat([buffer, chunk]);
            done();
        },
        async flush(done) {
            try {
                done(null, await (0, template_1.compile)(configuration, buffer.toString("utf8"), filepath));
                (0, debug_1.default)(`compiled ${filepath}`);
            }
            catch (e) {
                done(e);
            }
        },
    });
}
function preprendTransformerToOptions(configuration, options) {
    let wrappedTransform;
    if (!options.browserifyOptions ||
        !Array.isArray(options.browserifyOptions.transform)) {
        wrappedTransform = [transform.bind(null, configuration)];
    }
    else {
        wrappedTransform = [
            transform.bind(null, configuration),
            ...options.browserifyOptions.transform,
        ];
    }
    return Object.assign(Object.assign({}, options), { browserifyOptions: Object.assign(Object.assign({}, (options.browserifyOptions || {})), { transform: wrappedTransform }) });
}
